@@ -4,7 +4,7 @@ use warnings FATAL => 'all';
 
 package namespace::local;
 
-our $VERSION = '0.0604';
+our $VERSION = '0.07';
 
 =head1 NAME
 
@@ -156,35 +156,30 @@ work as expected if used simultaneously.
 
 =cut
 
-use Carp;
-
 # this was stolen from namespace::clean
 use B::Hooks::EndOfScope 'on_scope_end';
-use Scalar::Util qw( weaken );
 
-my $last_command;
+my @stack;
 
 sub import {
     my $class = shift;
 
-    my $command = namespace::local::_command->new( target => scalar caller );
+    my $command = namespace::local::_command->new( caller => [ caller ] );
     $command->parse_options( @_ );
 
     # on_scope_end callback execution order is direct
     # we need reversed order, so use a stack of commands.
-    $last_command->set_next( $command ) if $last_command;
-    $last_command = $command;
-    weaken $last_command; # to avoid leaks
+    $stack[-1]->set_next( $command ) if @stack;
+    push @stack, $command;
 
     $command->prepare;
 
     on_scope_end {
+        pop @stack;
         $command->execute;
     };
 };
 
-;
-
 # Hide internal OO engine
 # Maybe it will be released later...
 
@@ -199,8 +194,10 @@ sub new {
     my ($class, %opt) = @_;
 
     # TODO check options
-    $opt{except_rex} = qr/^[0-9]+$|^-$/; # no matter what, exempt $_, $1, ...
+    $opt{except_rex} = qr/^[0-9]+$|^_$/; # no matter what, exempt $_, $1, ...
     $opt{action} = '-around';
+    $opt{target} = $opt{caller}[0];
+    $opt{origin} = join ":", @{$opt{caller}}[1,2];
 
     # Skip some well-known variables and functions
     # Format: touch_not{ $name }{ $type }
@@ -215,9 +212,19 @@ sub new {
 
 sub set_next {
     my ($self, $next) = @_;
+
+    carp "probably a bug in namespace::local - uncommitted command replaced in chain"
+        if $self->{next} and !$self->{next}{done};
+
     $self->{next} = $next;
 };
 
+sub DESTROY {
+    my $self = shift;
+    carp "probably a bug in namespace::local: callback set but not executed"
+        if $self->{todo} and !$self->{done};
+};
+
 my %known_action;
 $known_action{$_}++ for qw(-above -below -around);
 
@@ -429,13 +436,13 @@ sub _croak {
     croak ("namespace::local: ".shift);
 };
 
-=head1 AUTHOR
-
-Konstantin S. Uvarin, C<< <khedin at gmail.com> >>
-
 =head1 BUGS
 
-This is experimental module. There certainly are bugs.
+As of 0.0604, C<-around> hides subroutines defined below its scope end
+from anything above it.
+No solution exists so far.
+
+This is experimental module. There certainly are more bugs.
 
 Bug reports, feature requests, suggestions and general feedback welcome at:
 
@@ -489,7 +496,7 @@ L<B::Hooks::EndOfScope> is used as a backend.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2018 Konstantin S. Uvarin.
+Copyright 2018 Konstantin S. Uvarin, C<< <khedin at gmail.com> >>
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the the Artistic License (2.0). You may obtain a
